@@ -1,4 +1,4 @@
-# This file is part of GNU Health.  The COPYRIGHT file at the top level of
+# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
@@ -72,7 +72,7 @@ class Record(SignalEvent):
                 fnames.append('rec_name')
             fnames.append('_timestamp')
 
-            record_context = self.context_get()
+            record_context = self.get_context()
             if loading == 'eager':
                 limit = int(CONFIG['client.limit'] / len(fnames))
 
@@ -84,7 +84,7 @@ class Record(SignalEvent):
                         and record.id not in id2record
                         and ((record.group == self.group)
                             # Don't compute context for same group
-                            or (record.context_get() == record_context)))
+                            or (record.get_context() == record_context)))
 
                 if self.parent and self.parent.model_name == self.model_name:
                     group = sum(self.parent.group.children, [])
@@ -251,7 +251,8 @@ class Record(SignalEvent):
     def get(self):
         value = {}
         for name, field in self.group.fields.iteritems():
-            if field.attrs.get('readonly'):
+            if (field.attrs.get('readonly')
+                    and not isinstance(field, fields.O2MField)):
                 continue
             if field.name not in self.modified_fields and self.id >= 0:
                 continue
@@ -299,7 +300,7 @@ class Record(SignalEvent):
         values = self._get_on_change_args(self.modified_fields)
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
-                context=self.context_get())
+                context=self.get_context())
         except RPCException:
             return False
         return True
@@ -310,7 +311,7 @@ class Record(SignalEvent):
                 value = self.get()
                 try:
                     res, = RPCExecute('model', self.model_name, 'create',
-                        [value], context=self.context_get())
+                        [value], context=self.get_context())
                 except RPCException:
                     return False
                 old_id = self.id
@@ -319,7 +320,7 @@ class Record(SignalEvent):
             elif self.modified:
                 value = self.get()
                 if value:
-                    context = self.context_get()
+                    context = self.get_context()
                     context = context.copy()
                     context['_timestamp'] = self.get_timestamp()
                     try:
@@ -365,7 +366,7 @@ class Record(SignalEvent):
 
     def default_get(self, rec_name=None):
         if len(self.group.fields):
-            context = self.context_get()
+            context = self.get_context()
             context.setdefault('default_rec_name', rec_name)
             try:
                 vals = RPCExecute('model', self.model_name, 'default_get',
@@ -382,16 +383,12 @@ class Record(SignalEvent):
                         == self.group.parent.model_name):
                     vals[self.parent_name] = self.parent.id
             self.set_default(vals)
-        for fieldname, fieldinfo in self.group.fields.iteritems():
-            if not fieldinfo.attrs.get('autocomplete'):
-                continue
-            self.do_autocomplete(fieldname)
         return vals
 
     def rec_name(self):
         try:
             return RPCExecute('model', self.model_name, 'read', [self.id],
-                ['rec_name'], context=self.context_get())[0]['rec_name']
+                ['rec_name'], context=self.get_context())[0]['rec_name']
         except RPCException:
             return ''
 
@@ -422,7 +419,7 @@ class Record(SignalEvent):
 
     invalid_fields = property(_get_invalid_fields)
 
-    def context_get(self):
+    def get_context(self):
         return self.group.context
 
     def set_default(self, val, signal=True, validate=True):
@@ -478,9 +475,6 @@ class Record(SignalEvent):
         for fieldname, value in later.iteritems():
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
-        for fieldname, fieldinfo in self.group.fields.iteritems():
-            if fieldinfo.attrs.get('autocomplete'):
-                self.do_autocomplete(fieldname)
         if validate:
             self.validate(fieldnames, softvalidation=True)
         if signal:
@@ -514,7 +508,7 @@ class Record(SignalEvent):
             return expr
         ctx = rpc.CONTEXT.copy()
         ctx['context'] = ctx.copy()
-        ctx['context'].update(self.context_get())
+        ctx['context'].update(self.get_context())
         ctx.update(self.get_eval())
         ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
@@ -550,7 +544,7 @@ class Record(SignalEvent):
         if values:
             try:
                 changes = RPCExecute('model', self.model_name, 'on_change',
-                    values, fieldnames, context=self.context_get())
+                    values, fieldnames, context=self.get_context())
             except RPCException:
                 return
             for change in changes:
@@ -581,7 +575,7 @@ class Record(SignalEvent):
         if fieldnames:
             try:
                 result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), context=self.context_get())
+                    values, list(fieldnames), context=self.get_context())
             except RPCException:
                 return
             self.set_on_change(result)
@@ -592,7 +586,7 @@ class Record(SignalEvent):
             try:
                 result = RPCExecute('model', self.model_name,
                     'on_change_with_' + fieldname, values,
-                    context=self.context_get())
+                    context=self.get_context())
             except RPCException:
                 return
             self.group.fields[fieldname].set_on_change(self, result)
@@ -610,7 +604,7 @@ class Record(SignalEvent):
         args = self._get_on_change_args(autocomplete)
         try:
             res = RPCExecute('model', self.model_name, 'autocomplete_' +
-                fieldname, args, context=self.context_get())
+                fieldname, args, context=self.get_context())
         except RPCException:
             # ensure res is a list
             res = []
@@ -635,7 +629,7 @@ class Record(SignalEvent):
                     'search_count', [
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
-                        ], context=self.context_get())
+                        ], context=self.get_context())
             except RPCException:
                 return 0
         return self.attachment_count
@@ -650,7 +644,7 @@ class Record(SignalEvent):
                         ('resource', '=',
                             '%s,%s' % (self.model_name, self.id)),
                         ('unread', '=', True),
-                        ], context=self.context_get())
+                        ], context=self.get_context())
             except RPCException:
                 return 0
         return self.unread_note
