@@ -1,4 +1,4 @@
-# This file is part of GNU Health.  The COPYRIGHT file at the top level of
+# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 "Screen"
 import copy
@@ -21,15 +21,16 @@ from tryton.gui.window.view_form.view.screen_container import ScreenContainer
 from tryton.gui.window.view_form.view import View
 from tryton.signal_event import SignalEvent
 from tryton.config import CONFIG
-from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
 from tryton.jsonrpc import JSONEncoder
 from tryton.common.domain_parser import DomainParser
 from tryton.common import RPCExecute, RPCException, MODELACCESS, \
     node_attributes, sur, RPCContextReload, warning
 from tryton.action import Action
+from tryton.pyson import PYSONDecoder
 import tryton.rpc as rpc
 
 _ = gettext.gettext
+logger = logging.getLogger(__name__)
 
 
 class Screen(SignalEvent):
@@ -39,41 +40,31 @@ class Screen(SignalEvent):
     # It is shared with all connection but it is the price for speed.
     tree_column_width = collections.defaultdict(lambda: {})
 
-    def __init__(self, model_name, view_ids=None, mode=None, context=None,
-            views_preload=None, domain=None, row_activate=None, limit=None,
-            readonly=False, exclude_field=None, order=None, search_value=None,
-            tab_domain=None, context_model=None, alternate_view=False):
-        if view_ids is None:
-            view_ids = []
-        if mode is None:
-            mode = ['tree', 'form']
-        if views_preload is None:
-            views_preload = {}
-        if domain is None:
-            domain = []
-
-        self.limit = limit or CONFIG['client.limit']
+    def __init__(self, model_name, **attributes):
+        context = attributes.get('context', {})
+        self.limit = attributes.get('limit', CONFIG['client.limit'])
         self.offset = 0
         super(Screen, self).__init__()
 
-        self.readonly = readonly
+        self.readonly = attributes.get('readonly', False)
         if not (MODELACCESS[model_name]['write']
                 or MODELACCESS[model_name]['create']):
             self.readonly = True
         self.search_count = 0
-        if not row_activate:
+        if not attributes.get('row_activate'):
             self.row_activate = self.default_row_activate
         else:
-            self.row_activate = row_activate
-        self.domain = domain
+            self.row_activate = attributes['row_activate']
+        self.domain = attributes.get('domain', [])
+        self.context_domain = attributes.get('context_domain')
         self.size_limit = None
-        self.views_preload = views_preload
+        self.views_preload = attributes.get('views_preload', {})
         self.model_name = model_name
         self.views = []
-        self.view_ids = view_ids[:]
+        self.view_ids = attributes.get('view_ids', [])[:]
         self.parent = None
         self.parent_name = None
-        self.exclude_field = exclude_field
+        self.exclude_field = attributes.get('exclude_field')
         self.filter_widget = None
         self.tree_states = collections.defaultdict(
             lambda: collections.defaultdict(lambda: None))
@@ -82,14 +73,15 @@ class Screen(SignalEvent):
         self.new_group(context or {})
         self.__current_record = None
         self.current_record = None
-        self.screen_container = ScreenContainer(tab_domain)
-        self.screen_container.alternate_view = alternate_view
+        self.screen_container = ScreenContainer(attributes.get('tab_domain'))
+        self.screen_container.alternate_view = attributes.get(
+            'alternate_view', False)
         self.widget = self.screen_container.widget_get()
 
         self.context_screen = None
-        if context_model:
+        if attributes.get('context_model'):
             self.context_screen = Screen(
-                context_model, mode=['form'], context=context)
+                attributes['context_model'], mode=['form'], context=context)
             self.context_screen.new()
             context_widget = self.context_screen.widget
 
@@ -134,14 +126,19 @@ class Screen(SignalEvent):
             self.context_screen.widget.show()
 
         self.__current_view = 0
-        self.search_value = search_value
+        self.search_value = attributes.get('search_value')
         self.fields_view_tree = {}
-        self.order = self.default_order = order
+        self.order = self.default_order = attributes.get('order')
+        self.__date_format = self.context.get(
+            'date_format', rpc.CONTEXT.get('locale', {}).get('date', '%x'))
         self.view_to_load = []
         self._domain_parser = {}
         self.pre_validate = False
+        mode = attributes.get('mode')
+        if mode is None:
+            mode = ['tree', 'form']
         self.view_to_load = mode[:]
-        if view_ids or mode:
+        if self.view_ids or self.view_to_load:
             self.switch_view()
         self.count_tab_domain()
 
@@ -230,11 +227,13 @@ class Screen(SignalEvent):
         return selection
 
     def search_prev(self, search_string):
-        self.offset -= self.limit
+        if self.limit:
+            self.offset -= self.limit
         self.search_filter(search_string=search_string)
 
     def search_next(self, search_string):
-        self.offset += self.limit
+        if self.limit:
+            self.offset += self.limit
         self.search_filter(search_string=search_string)
 
     def search_complete(self, search_string):
@@ -252,6 +251,9 @@ class Screen(SignalEvent):
             self.new_group(context)
 
         domain = self.search_domain(search_string, True)
+        if self.context_domain:
+            decoder = PYSONDecoder(self.context)
+            domain = ['AND', domain, decoder.decode(self.context_domain)]
         tab_domain = self.screen_container.get_tab_domain()
         if tab_domain:
             domain = ['AND', domain, tab_domain]
@@ -262,7 +264,7 @@ class Screen(SignalEvent):
         except RPCException:
             ids = []
         if not only_ids:
-            if len(ids) == self.limit:
+            if self.limit is not None and len(ids) == self.limit:
                 try:
                     self.search_count = RPCExecute('model', self.model_name,
                         'search_count', domain, context=self.context)
@@ -271,7 +273,8 @@ class Screen(SignalEvent):
             else:
                 self.search_count = len(ids)
         self.screen_container.but_prev.set_sensitive(bool(self.offset))
-        if (len(ids) == self.limit
+        if (self.limit is not None
+                and len(ids) == self.limit
                 and self.search_count > self.limit + self.offset):
             self.screen_container.but_next.set_sensitive(True)
         else:
@@ -312,7 +315,11 @@ class Screen(SignalEvent):
 
     def count_tab_domain(self):
         def set_tab_counter(count, idx):
-            self.screen_container.set_tab_counter(count(), idx)
+            try:
+                count = count()
+            except RPCException:
+                count = None
+            self.screen_container.set_tab_counter(count, idx)
         screen_domain = self.search_domain(self.screen_container.get_text())
         for idx, (name, domain, count) in enumerate(
                 self.screen_container.tab_domain):
@@ -328,6 +335,10 @@ class Screen(SignalEvent):
     def context(self):
         return self.group.context
 
+    @property
+    def date_format(self):
+        return self.__date_format
+
     def __get_group(self):
         return self.__group
 
@@ -379,6 +390,7 @@ class Screen(SignalEvent):
     def _group_changed(self, group, record):
         if not self.parent:
             self.display()
+        self.signal('group-changed', record)
 
     def __get_current_record(self):
         if (self.__current_record is not None
@@ -430,6 +442,7 @@ class Screen(SignalEvent):
         return False
 
     def destroy(self):
+        self.screen_container.destroy()
         for view in self.views:
             view.destroy()
         del self.views[:]
@@ -588,7 +601,7 @@ class Screen(SignalEvent):
         if self.current_view.view_type == 'tree':
             # False value must be not saved
             saved = all((x is 0 or x > 0 for x in self.group.save()))
-            record_id = self.current_record.id
+            record_id = self.current_record.id if self.current_record else None
         elif self.current_record.validate(fields):
             record_id = self.current_record.save(force_reload=True)
             # False value must be not saved
@@ -749,8 +762,9 @@ class Screen(SignalEvent):
                 expanded_nodes = json.loads(expanded_nodes)
                 selected_nodes = json.loads(selected_nodes)
             except RPCException:
-                logging.getLogger(__name__).warn(
-                    _('Unable to get view tree state'))
+                logger.warn(
+                    _('Unable to get view tree state for %s')
+                    % self.model_name)
             self.tree_states[parent][view.children_field] = (
                 timestamp, expanded_nodes, selected_nodes)
         if view.view_type == 'tree':
@@ -800,23 +814,25 @@ class Screen(SignalEvent):
                     timestamp, paths, selected_paths)
                 if store and view.attributes.get('tree_state', False):
                     json_domain = self.get_tree_domain(parent)
-                    json_paths = json.dumps(paths)
-                    json_selected_path = json.dumps(selected_paths)
+                    json_paths = json.dumps(paths, separators=(',', ':'))
+                    json_selected_path = json.dumps(
+                        selected_paths, separators=(',', ':'))
                     try:
                         RPCExecute('model', 'ir.ui.view_tree_state', 'set',
                             self.model_name, json_domain, view.children_field,
                             json_paths, json_selected_path,
                             process_exception=False)
-                    except (TrytonServerError, TrytonServerUnavailable):
-                        logging.getLogger(__name__).warn(
-                            _('Unable to set view tree state'))
+                    except Exception:
+                        logger.warn(
+                            _('Unable to set view tree state'), exc_info=True)
 
     def get_tree_domain(self, parent):
         if parent:
             domain = (self.domain + [(self.exclude_field, '=', parent)])
         else:
             domain = self.domain
-        json_domain = json.dumps(domain, cls=JSONEncoder)
+        json_domain = json.dumps(
+            domain, cls=JSONEncoder, separators=(',', ':'))
         return json_domain
 
     def load(self, ids, set_cursor=True, modified=False):
@@ -1021,8 +1037,6 @@ class Screen(SignalEvent):
     def get_buttons(self):
         'Return active buttons for the current view'
         def is_active(record, button):
-            if record.group.readonly or record.readonly:
-                return False
             if button.attrs.get('type', 'class') == 'instance':
                 return False
             states = record.expr_eval(button.attrs.get('states', {}))
@@ -1094,7 +1108,7 @@ class Screen(SignalEvent):
                     'model': self.model_name,
                     'id': self.current_record.id,
                     'ids': ids,
-                    }, context=self.context)
+                    }, context=self.context, keyword=True)
 
     def client_action(self, action):
         access = MODELACCESS[self.model_name]
@@ -1131,28 +1145,36 @@ class Screen(SignalEvent):
         elif action == 'reload context':
             RPCContextReload()
 
-    def get_url(self):
+    def get_url(self, name=''):
         query_string = []
         if self.domain:
-            query_string.append(('domain', json.dumps(self.domain,
-                        cls=JSONEncoder)))
+            query_string.append(('domain', json.dumps(
+                        self.domain, cls=JSONEncoder, separators=(',', ':'))))
         if self.context:
-            query_string.append(('context', json.dumps(self.context,
-                        cls=JSONEncoder)))
+            query_string.append(('context', json.dumps(
+                        self.context, cls=JSONEncoder, separators=(',', ':'))))
+        if self.context_screen:
+            query_string.append(
+                ('context_model', self.context_screen.model_name))
+        if name:
+            query_string.append(
+                ('name', json.dumps(name, separators=(',', ':'))))
         path = [rpc._DATABASE, 'model', self.model_name]
         view_ids = [v.view_id for v in self.views] + self.view_ids
         if self.current_view.view_type != 'form':
             search_string = self.screen_container.get_text()
             search_value = self.domain_parser.parse(search_string)
             if search_value:
-                query_string.append(('search_value', json.dumps(search_value,
-                            cls=JSONEncoder)))
+                query_string.append(('search_value', json.dumps(
+                            search_value, cls=JSONEncoder,
+                            separators=(',', ':'))))
         elif self.current_record and self.current_record.id > -1:
             path.append(str(self.current_record.id))
             i = view_ids.index(self.current_view.view_id)
             view_ids = view_ids[i:] + view_ids[:i]
         if view_ids:
-            query_string.append(('views', json.dumps(view_ids)))
+            query_string.append(('views', json.dumps(
+                        view_ids, separators=(',', ':'))))
         query_string = urllib.urlencode(query_string)
         return urlparse.urlunparse(('tryton',
                 '%s:%s' % (rpc._HOST, rpc._PORT),
