@@ -1,4 +1,4 @@
-# This file is part of GNU Health.  The COPYRIGHT file at the top level of
+# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import gtk
 import gobject
@@ -27,8 +27,7 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
         child.connect('focus-out-event', lambda *a: self._focus_out())
         self.entry.connect('changed', self.changed)
         self.entry.connect('move-active', self._move_active)
-        self.entry.connect(
-            'scroll-event', lambda c, e: c.emit_stop_by_name('scroll-event'))
+        self.entry.connect('scroll-event', self._scroll_event)
         self.widget.pack_start(self.entry)
         self.widget.set_focus_chain([child])
 
@@ -48,6 +47,10 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
         if not combobox.get_child().get_editable():
             combobox.emit_stop_by_name('move-active')
 
+    def _scroll_event(self, combobox, event):
+        if not combobox.get_child().get_editable():
+            combobox.emit_stop_by_name('scroll-event')
+
     def _readonly_set(self, value):
         super(Selection, self)._readonly_set(value)
         self.entry.get_child().set_editable(not value)
